from plone.multilingual.interfaces import ILanguage, ITranslationManager


# unregister the translation before the object will be removed
def remove_translation_on_delete(obj, event):
    language = ILanguage(obj).get_language()
    ITranslationManager(obj).remove_translation(language)


# update the translation on Modified (because the language could be modified)
def update_on_modify(obj, event):
    ITranslationManager(obj).update()



from zope.interface import Interface, Attribute


# CONSTANTS
LANGUAGE_INDEPENDENT = 'no_language'


# Language-support
class ILanguage(Interface):

    def get_language(self):
        """ return the contents language """

    def set_language(self):
        """ return the contents language """


# Marker interface
class ITranslatable(Interface):
    """Marker interface for content types that support translation"""


# adapters
class ITranslationFactory(Interface):
    """Adapts ITranslated and is capable of returning
       a translation clone to be added.
    """

    def __call__(language):
        """Create a clone of the context
           for translation to the given language
        """


class ITranslationLocator(Interface):
    """Find a parent folder for a translation.
       Adapts ITranslated.
    """

    def __call__(language):
        """Return a parent folder into which a new translation can be added"""


class ITranslationIdChooser(Interface):
    """Find a valid id for a translation
       Adapts ITranslated.
    """

    def __call__(parent, language):
        """ Return a valid id for the translation """


class ITranslationCloner(Interface):
    """Subscription adapters to perform various aspects of cloning an object.
       Allows componentisation of things like workflow history cloning.
       Adapts ITranslated.
    """

    def __call__(object):
        """Update the translation copy that is being constructed"""


class ICanonical(Interface):

    languages = Attribute(u"dictionary {LANG_KEY: UUID, ...}")

    def add_item(language, intid):
        """ """

    def remove_item(language):
        """ """

    def get_item(language):
        """ """

    def remove_item_by_language(language):
        """ """

    def remove_item_by_id(id):
        """ """

    def get_items():
        """ """

    def get_keys():
        """ """


class ITranslationManager(Interface):

    def add_translation(object, intid):
        """
        create the translated content and register the translation
        """

    def remove_translation(language):
        """
        remove translation if exists (unregister the translation)
        """

    def get_translation(language):
        """
        get translation (translated object) if exists
        """

    def get_translations():
        """
        get all the translated objects (including the context)
        """

    def get_translated_languages():
        """
        get a list of the translated languages
        (language-code like 'en', 'it' etc. )
        """

    def register_translation(language, content):
        """
        register an existing content as translation
        for context
        """

    def update():
        """
        update the item registered in the canonical
        (used for changing the contexts language)
        """

    def query_canonical():
        """
        query if there is an canonical for the context
        used for migration
        """


class IMultilingualStorage(Interface):
    """ Stores the canonical objects at the portal_multilingual tool """

    def add_canonical(id, canonical):
        """ add canonical """

    def get_canonical(id):
        """ get canonical """

    def remove_canonical(id):
        """ remove canonical """


class ICanonicalStorage(Interface):
    """ Deprecated in 0.2, for migration purposes only """

    def add_canonical(id, canonical):
        """ add canonical """

    def get_canonical(id):
        """ get canonical """

    def remove_canonical(id):
        """ remove canonical """


class ILanguageIndependentFieldsManager(Interface):
    context = Attribute("context", "A translatable object")

    def copy_fields(translation):
        """ Copy language independent fields to translation."""





# -*- coding: utf-8 -*-
import doctest
from plone.app.testing import (
    PLONE_FIXTURE,
    PloneSandboxLayer,
    applyProfile,
    IntegrationTesting,
    FunctionalTesting,
)
from zope.configuration import xmlconfig
from zope.interface import implements
from zope.component import adapts
from plone.multilingual.interfaces import ITranslatable, ILanguage
from plone.uuid.interfaces import IAttributeUUID


class DemoLanguage(object):

    implements(ILanguage)
    adapts(ITranslatable)

    def __init__(self, context):
        self.context = context

    def get_language(self):
        return self.context.lang

    def set_language(self, lang):
        self.context.lang = lang


class PloneMultilingualLayer(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import plone.uuid
        import plone.multilingual

        # load ZCML
        xmlconfig.file('configure.zcml', plone.multilingual,
                        context=configurationContext)
        xmlconfig.file('configure.zcml', plone.multilingual.tests,
                        context=configurationContext)

    def setUpPloneSite(self, portal):
        # install into the Plone site
        applyProfile(portal, 'plone.multilingual:default')
        from zope.interface import classImplements
        from Products.ATContentTypes.content.folder import ATFolder
        classImplements(ATFolder, ITranslatable)


PLONEMULTILINGUAL_FIXTURE = PloneMultilingualLayer()
PLONEMULTILINGUAL_INTEGRATION_TESTING = IntegrationTesting(\
    bases=(PLONEMULTILINGUAL_FIXTURE,), name="plone.multilingual:Integration")
PLONEMULTILINGUAL_FUNCTIONAL_TESTING = FunctionalTesting(\
    bases=(PLONEMULTILINGUAL_FIXTURE,), name="plone.multilingual:Functional")
optionflags = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)



from zope import interface
from zope.event import notify
from zope.component import getUtility
from plone.uuid.interfaces import IUUID
from plone.multilingual.interfaces import (
    ILanguage,
    ITranslationManager,
    ITranslationFactory,
    IMultilingualStorage,
)
from plone.multilingual.canonical import Canonical
from plone.multilingual.events import (
    ObjectWillBeTranslatedEvent,
    ObjectTranslatedEvent,
)
from plone.uuid.handlers import addAttributeUUID
from plone.app.uuid.utils import uuidToObject


class TranslationManager(object):

    interface.implements(ITranslationManager)

    def get_id(self, context):
        """If an object is created via portal factory we don't get a id, we
           have to wait till the object is really created.
           TODO: a better check if we are in the portal factory!
        """
        try:
            context_id = IUUID(context)
        # We must ensure that this case can't happen, any object translatable
        # will have an UUID (in any case we can be at the portal factory!)
        except KeyError, e:
            addAttributeUUID(context, None)
            context.reindexObject(idxs=['UID'])
            context_id = IUUID(context)
        return context_id

    def __init__(self, context):
        self.context = context

    def _get_or_create(self):
        """ get the canonical for context
            (create it, if it's not existing)
        """
        canonical = self._get_canonical()
        if canonical is None:
            canonical = self._add_canonical()
        return canonical

    def _get_canonical(self):
        """ get the canonical for context """
        storage = getUtility(IMultilingualStorage)
        context_id = self.get_id(self.context)
        canonical = storage.get_canonical(context_id)
        return canonical

    def _add_canonical(self):
        """ create the cononical for context """
        storage = getUtility(IMultilingualStorage)
        context_id = self.get_id(self.context)
        canonical = Canonical()
        canonical.add_item(ILanguage(self.context).get_language(), context_id)
        storage.add_canonical(context_id, canonical)
        return canonical

    def query_canonical(self):
        canonical = self._get_canonical()
        return canonical

    def register_translation(self, language, content):
        """ register a translation for an existing content """
        canonical = self._get_or_create()
        if type(content) == str:
            content_id = content
        else:
            content_id = self.get_id(content)

        # register the new translation in the canonical
        canonical.add_item(language, content_id)

        # register the canonical for the translated object
        storage = getUtility(IMultilingualStorage)
        storage.add_canonical(content_id, canonical)
        return

    def update(self):
        """ see interfaces"""
        language = ILanguage(self.context).get_language()
        if language not in self.get_translated_languages():
            canonical = self._get_or_create()
            content_id = self.get_id(self.context)
            canonical.remove_item_by_id(content_id)
            canonical.add_item(language, content_id)

    def add_translation(self, language):
        """ see interfaces """
        if not language:
            raise KeyError('There is no target language')
        # event
        notify(ObjectWillBeTranslatedEvent(self.context, language))
        # create the translated object
        translation_factory = ITranslationFactory(self.context)
        translated_object = translation_factory(language)
        ILanguage(translated_object).set_language(language)
        # reindex the translated object
        translated_object.reindexObject()
        # register the new translation
        translated_id = self.get_id(translated_object)
        self.register_translation(language, translated_id)
        # event
        notify(ObjectTranslatedEvent(self.context, \
            translated_object, language))
        return

    def remove_translation(self, language):
        """ see interfaces """
        canonical = self._get_canonical()
        if canonical is not None:
            translated_id = canonical.get_item(language)
            # remove language from canonical
            canonical.remove_item_by_language(language)
            # unregister the canonical for the translated object
            storage = getUtility(IMultilingualStorage)
            storage.remove_canonical(translated_id)

    def get_translation(self, language):
        """ see interfaces """
        translation = None
        canonical = self._get_canonical()
        if canonical is not None:
            translation = uuidToObject(canonical.get_item(language))
        return translation

    def get_translations(self):
        """ see interfaces """
        translations = {}
        canonical = self._get_canonical()
        if canonical is not None:
            for language, content_id in canonical.get_items():
                translations[language] = uuidToObject(content_id)
        return translations

    def get_translated_languages(self):
        """ see interfaces """
        languages = []
        canonical = self._get_or_create()
        if canonical is not None:
            languages = canonical.get_keys()
        return languages

    def has_translation(self, language):
        """ see interfaces """
        return language in self.get_translated_languages()



from zope.container.contained import Contained
from zope.interface import implements
from persistent import Persistent
from persistent.dict import PersistentDict
from plone.multilingual.interfaces import ICanonical


class Canonical(Persistent, Contained):
    """
    Canonical object that is stored on the Canonicals-Storage
    """
    implements(ICanonical)

    @property
    def languages(self):
        if getattr(self, '_languages', None) is None:
            self._languages = PersistentDict()
        return self._languages

    def add_item(self, language, id):
        if language in self.languages.keys():
            raise KeyError("Translation already exists")
        if not language:
            raise KeyError("There is no target language")
        self.languages[language] = id

    def remove_item_by_language(self, language):
        if language in self.languages.keys():
            del self.languages[language]

    def remove_item_by_id(self, id):
        language = None
        for item_language, item_id in self.languages.items():
            if item_id == id:
                language = item_language
                break
        if language is not None:
            del(self.languages[language])

    def get_item(self, language):
        item = None
        if language in self.languages.keys():
            item = self.languages[language]
        return item

    def get_items(self):
        return self.languages.items()

    def get_keys(self):
        return self.languages.keys()
